@@ -19,11 +19,11 @@ Class::Sniff - Look for class composition code smells
 
 =head1 VERSION
 
-Version 0.05
+Version 0.06
 
 =cut
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 =head1 SYNOPSIS
 
@@ -104,6 +104,12 @@ Optional.
 If present and true, will attempt to include the C<UNIVERSAL> base class.  If
 a class hierarchy is pruned with C<ignore>, C<UNIVERSAL> may not show up.
 
+=item * method_length
+
+If present, will set the "maximum length" of a method before it's reported as
+a code smell.  This feature is I<highly> experimental.  See C<long_methods>
+for details.
+
 =back
 
 =cut
@@ -116,18 +122,20 @@ sub new {
         Carp::croak("'ignore' requires a regex");
     }
     my $self = bless {
-        class_order  => {},
-        classes      => {},
-        duplicates   => {},
-        exported     => {},
-        graph        => undef,
-        ignore       => $arg_for->{ignore},
-        list_classes => [$target_class],
-        methods      => {},
-        paths        => [ [$target_class] ],
-        target       => $target_class,
-        tree         => undef,
-        universal    => $arg_for->{universal},
+        class_order   => {},
+        classes       => {},
+        duplicates    => {},
+        exported      => {},
+        graph         => undef,
+        ignore        => $arg_for->{ignore},
+        list_classes  => [$target_class],
+        long_methods  => {},
+        methods       => {},
+        paths         => [ [$target_class] ],
+        target        => $target_class,
+        tree          => undef,
+        universal     => $arg_for->{universal},
+        method_length => ( $arg_for->{method_length} || 50 ),
     } => $class;
     $self->_initialize;
     return $self;
@@ -137,7 +145,7 @@ sub _initialize {
     my $self         = shift;
     my $target_class = $self->target_class;
     $self->width(72);
-    $self->_add_class($target_class);
+    $self->_register_class($target_class);
     $self->{classes}{$target_class}{count} = 1;
     $self->{tree} = Tree->new($target_class);
     $self->_build_tree( $self->tree );
@@ -166,7 +174,7 @@ sub _finalize {
     $self->{class_order} = \%classes;
 }
 
-sub _add_class {
+sub _register_class {
     my ( $self, $class ) = @_;
     return if exists $self->{classes}{$class};
 
@@ -181,6 +189,15 @@ sub _add_class {
         if ( $info->package ne $class ) {
             $self->{exported}{$class}{$method} = $info->package;
         }
+        else {
+            eval {
+                my $line = $info->line;
+                my $length  = B::svref_2object($coderef)->GV->LINE - $line;
+                if ( $length > $self->method_length ) {
+                    $self->{long_methods}{"$class\::$method"} = $length;
+                }
+            };
+        }
 
         my $walker = B::Concise::compile( '-terse', $coderef );    # 1
         B::Concise::walk_output( \my $buffer);
@@ -206,206 +223,7 @@ sub _add_class {
     return $self;
 }
 
-=head1 INSTANCE METHODS
-
-=head2 C<report>
-
- print $sniff->report;
-
-Prints out a detailed, human readable report of C<Class::Sniff>'s analysis of
-the class.  Returns an empty string if no issues found.  Sample:
-
- Report for class: Grandchild
- 
- Overridden Methods
- .--------+--------------------------------------------------------------------.
- | Method | Class                                                              |
- +--------+--------------------------------------------------------------------+
- | bar    | Grandchild                                                         |
- |        | Abstract                                                           |
- |        | Child2                                                             |
- | foo    | Grandchild                                                         |
- |        | Child1                                                             |
- |        | Abstract                                                           |
- |        | Child2                                                             |
- '--------+--------------------------------------------------------------------'
- Unreachable Methods
- .--------+--------------------------------------------------------------------.
- | Method | Class                                                              |
- +--------+--------------------------------------------------------------------+
- | bar    | Child2                                                             |
- | foo    | Child2                                                             |
- '--------+--------------------------------------------------------------------'
- Multiple Inheritance
- .------------+----------------------------------------------------------------.
- | Class      | Parents                                                        |
- +------------+----------------------------------------------------------------+
- | Grandchild | Child1                                                         |
- |            | Child2                                                         |
- '------------+----------------------------------------------------------------'
-
-=cut
-
-sub report {
-    my $self = shift;
-
-    my $report = $self->_get_overridden_report;
-    $report .= $self->_get_unreachable_report;
-    $report .= $self->_get_multiple_inheritance_report;
-    $report .= $self->_get_exported_report;
-    $report .= $self->_get_duplicate_method_report;
-
-    if ($report) {
-        my $target = $self->target_class;
-        $report = "Report for class: $target\n\n$report";
-    }
-    return $report;
-}
-
-sub _get_duplicate_method_report {
-    my $self = shift;
-
-    my $report = '';
-    my @duplicate = $self->duplicate_methods;
-    my (@methods, @duplicates);
-    if ( @duplicate ) {
-        foreach my $duplicate (@duplicate) {
-            push @methods => join '::' => @{ pop @$duplicate };
-            push @duplicates => join "\n" => map { join '::' => @$_ } @$duplicate;
-        }
-        $report .= "Duplicate Methods (Experimental)\n"
-          . $self->_build_report( 'Method', 'Duplicated In', \@methods,
-          \@duplicates );
-    }
-    return $report;
-}
-
-sub _get_overridden_report {
-    my $self = shift;
-
-    my $report = '';
-    my $overridden = $self->overridden;
-    if ( %$overridden ) {
-        my @methods = sort keys %$overridden;
-        my @classes;
-        foreach my $method (@methods) {
-            push @classes => join "\n" => @{ $overridden->{$method} };
-        }
-        $report .= "Overridden Methods\n"
-          . $self->_build_report( 'Method', 'Class', \@methods, \@classes );
-    }
-    return $report;
-}
-
-sub _get_unreachable_report {
-    my $self = shift;
-
-    my $report = '';
-    if ( my @unreachable = $self->unreachable ) {
-        my ( @methods, @classes );
-        for my $fq_method (@unreachable) {
-            $fq_method =~ /^(.*)::(.*)$/;    # time to rethink the API
-            push @methods => $2;
-            push @classes => $1;
-        }
-        $report .= "Unreachable Methods\n"
-          . $self->_build_report( 'Method', 'Class', \@methods, \@classes );
-    }
-    return $report;
-}
-
-sub _get_multiple_inheritance_report {
-    my $self = shift;
-    my $report .= '';
-    if ( my @multis = $self->multiple_inheritance ) {
-        my @classes = map { join "\n" => $self->parents($_) } @multis;
-        $report .= "Multiple Inheritance\n"
-          . $self->_build_report( 'Class', 'Parents', \@multis, \@classes );
-    }
-    return $report;
-}
-
-sub _get_exported_report {
-    my $self = shift;
-    my $exported = $self->exported;
-    my $report = '';
-    if ( my @classes = sort keys %$exported ) {
-        my ($longest_c, $longest_m) = (length('Class'), length('Method') );
-        my (@subs,@sources);
-        foreach my $class (@classes) {
-            my (@temp_subs, @temp_sources);
-            foreach my $sub (sort keys %{ $exported->{$class} } ) {
-                push @temp_subs => $sub;
-                push @temp_sources => $exported->{$class}{$sub};
-                $longest_c = length($class) if length($class) > $longest_c;
-                $longest_m = length($sub)   if length($sub) > $longest_m;
-            }
-            push @subs    => join "\n" => @temp_subs;
-            push @sources => join "\n" => @temp_sources;
-        }
-        my $width = $self->width - 3;
-        my $third = int($width/3);
-        $longest_c = $third if $longest_c > $third;
-        $longest_m = $third if $longest_m > $third;
-        my $rest = $width - ($longest_c + $longest_m);
-        my $text = Text::SimpleTable->new(
-            [ $longest_c, 'Class' ],
-            [ $longest_m, 'Method' ],
-            [ $rest,      'Exported From Package' ]
-        );
-        for my $i ( 0 .. $#classes ) {
-            $text->row( $classes[$i], $subs[$i], $sources[$i] );
-        }
-        $report .= "Exported Subroutines\n".$text->draw;
-    }
-    return $report;
-}
-
-sub _build_report {
-    my ( $self, $title1, $title2, $strings1, $strings2 ) = @_;
-    unless ( @$strings1 == @$strings2 ) {
-        Carp::croak("PANIC:  Attempt to build unbalanced report");
-    }
-    my ( $width1, $width2 ) = $self->_get_widths( $title1, @$strings1 );
-    my $text =
-      Text::SimpleTable->new( [ $width1, $title1 ], [ $width2, $title2 ] );
-    for my $i ( 0 .. $#$strings1 ) {
-        $text->row( $strings1->[$i], $strings2->[$i] );
-    }
-    return $text->draw;
-}
-
-sub _get_widths {
-    my ( $self, $title, @strings ) = @_;
-
-    my $width = $self->width;
-    my $longest = length($title);
-    foreach my $string (@strings) {
-        my $length = length $string;
-        $longest = $length if $length > $longest;
-    }
-    $longest = int( $width / 2 ) if $longest > ($width / 2);
-    return ($longest, $width - $longest);
-}
-
-=head2 C<width>
-
- $sniff->width(80);
-
-Set the width of the report.  Defaults to 72.
-
-=cut
-
-sub width {
-    my $self = shift;
-    return $self->{width} unless @_;
-    my $number = shift;
-    unless ( $number =~ /^\d+$/ && $number >= 40 ) {
-        Carp::croak(
-            "Argument to 'width' must be a number >= than 40, not ($number)");
-    }
-    $self->{width} = $number;
-}
+=head1 INSTANCE METHODS - CODE SMELLS
 
 =head2 C<overridden>
 
@@ -422,6 +240,10 @@ Overridden methods are not necessarily a code smell, but you should check them
 to find out if you've overridden something you didn't expect to override.
 Accidental overriding of a method can be very hard to debug.
 
+This can also be a sign of bad responsibilities.  If you have a long
+inheritance chain and you override a method in five different levels with five
+different behaviors, perhaps this behavior should be in its own class.
+
 =cut
 
 sub overridden {
@@ -485,38 +307,37 @@ sub unreachable {
     my $self       = shift;
     my $overridden = $self->overridden;
     my @paths      = $self->paths;
-    my %unreachable;
 
+    # If we only have one path through our code, we don't have any unreachable
+    # methods.
+    return if @paths == 1;
+
+    # Algorithm:  If we have overridden methods, then if we have multiple
+    # paths through the code, a method is unreachable if a *previous* path
+    # contains the method because Perl's default search order won't get to
+    # successive paths.
+    my @unreachable;
     while ( my ( $method, $classes ) = each %$overridden ) {
-        my @unreachable;
+        my @classes;
 
       CLASS:
         for my $class (@$classes) {
             my $method_found = 0;
             for my $path (@paths) {
+
+                # method was found in a *previous* path.
                 if ($method_found) {
-                    push @unreachable => $class;
+                    push @unreachable => "$class\::$method";
                     next CLASS;
                 }
                 for my $curr_class (@$path) {
-                    if ( $curr_class eq $class ) {
-                        next CLASS;
-                    }
+                    next CLASS if $curr_class eq $class;
                     if ( not $method_found && $curr_class->can($method) ) {
                         $method_found = 1;
                     }
                 }
             }
         }
-        if (@unreachable) {
-            $unreachable{$method} = \@unreachable;
-        }
-    }
-    my @unreachable;
-    while ( my ( $method, $classes ) = each %unreachable ) {
-        foreach my $class (@$classes) {
-            push @unreachable => "$class\::$method";
-        }
     }
     return @unreachable;
 }
@@ -651,23 +472,309 @@ sub duplicate_methods {
     return @duplicates;
 }
 
-sub _add_relationships {
-    my ( $self, $class, @parents ) = @_;
-    $self->_add_class($_) foreach $class, @parents;
+=head2 C<long_methods> (highly experimental)
 
-    # what if this is called more than once?
-    $self->{classes}{$class}{parents} = \@parents;
-    $self->_add_child( $_, $class ) foreach @parents;
-    return $self;
+ my $num_long_methods = $sniff->long_methods;
+ my %long_methods     = $sniff->long_methods;
+
+Returns methods longer than C<method_length>.  This value defaults to 50 and
+can be overridden in the constructor (but not later).
+
+Caveats:  this is experimental and depends on C<Sub::Information> 0.10 and the
+ill-documented C<B> modules.  Specifically, it relies on the following:
+
+=over 4
+
+=item * How to count the length of a method.
+
+ my $start_line = B::svref_2object($coderef)->START->line;
+ my $end_line   = B::svref_2object($coderef)->GV->LINE;
+ my $method_length = $end_line - $start_line;
+
+The C<$start_line> returns the line number of the I<first expression> in the
+subroutine, not the C<sub foo { ...> declaration.  The subroutine's
+declaration actually ends at the ending curly brace, so the following method
+would be considered 3 lines long, even though you might count it differently:
+
+ sub new {
+     # this is our constructor
+     my ( $class, $arg_for ) = @_;
+     my $self = bless {} => $class;
+     return $self;
+ }
+
+=cut
+
+sub long_methods { %{ $_[0]->{long_methods} } }
+
+=item * Exported methods
+
+These are simply ignored because the C<B> modules think they start and end in
+different packages.
+
+=item * Where does it really start?
+
+If you've taken a reference to a method I<prior> to the declaration of the
+reference being seen, Perl might report a negative length or simply blow up.
+We trap that for you and you'll never see those.
+
+=back
+
+Let me know how it works out :)
+
+=head3 Code Smell:  long methods
+
+Long methods are probably doing to much and should be broken down into smaller
+methods.  They're harder to follow, harder to debug, and if they're doing more
+than one thing, you might find that you need that functionality elsewhere, but
+now it's tightly coupled to the long method's behavior.  As always, use your
+judgment.
+
+=head2 C<parents>
+
+ # defaults to 'target_class'
+ my $num_parents = $sniff->parents;
+ my @parents     = $sniff->parents;
+
+ my $num_parents = $sniff->parents('Some::Class');
+ my @parents     = $sniff->parents('Some::Class');
+
+In scalar context, lists the number of parents a class has.
+
+In list context, lists the parents a class has.
+
+=head3 Code Smell:  multiple parens (multiple inheritance)
+
+If a class has more than one parent, you may have unreachable or conflicting
+methods.
+
+=cut
+
+sub parents {
+    my ( $self, $class ) = @_;
+    $class ||= $self->target_class;
+    unless ( exists $self->{classes}{$class} ) {
+        Carp::croak("No such class '$class' found in hierarchy");
+    }
+    return @{ $self->{classes}{$class}{parents} };
 }
 
-sub _add_child {
-    my ( $self, $class, $child ) = @_;
+=head1 INSTANCE METHODS - REPORTING
 
-    my $children = $self->{classes}{$class}{children};
-    unless ( grep { $child eq $_ } @$children ) {
-        push @$children => $child;
+=head2 C<report>
+
+ print $sniff->report;
+
+Prints out a detailed, human readable report of C<Class::Sniff>'s analysis of
+the class.  Returns an empty string if no issues found.  Sample:
+
+ Report for class: Grandchild
+ 
+ Overridden Methods
+ .--------+--------------------------------------------------------------------.
+ | Method | Class                                                              |
+ +--------+--------------------------------------------------------------------+
+ | bar    | Grandchild                                                         |
+ |        | Abstract                                                           |
+ |        | Child2                                                             |
+ | foo    | Grandchild                                                         |
+ |        | Child1                                                             |
+ |        | Abstract                                                           |
+ |        | Child2                                                             |
+ '--------+--------------------------------------------------------------------'
+ Unreachable Methods
+ .--------+--------------------------------------------------------------------.
+ | Method | Class                                                              |
+ +--------+--------------------------------------------------------------------+
+ | bar    | Child2                                                             |
+ | foo    | Child2                                                             |
+ '--------+--------------------------------------------------------------------'
+ Multiple Inheritance
+ .------------+----------------------------------------------------------------.
+ | Class      | Parents                                                        |
+ +------------+----------------------------------------------------------------+
+ | Grandchild | Child1                                                         |
+ |            | Child2                                                         |
+ '------------+----------------------------------------------------------------'
+
+=cut
+
+sub report {
+    my $self = shift;
+
+    my $report = $self->_get_overridden_report;
+    $report .= $self->_get_unreachable_report;
+    $report .= $self->_get_multiple_inheritance_report;
+    $report .= $self->_get_exported_report;
+    $report .= $self->_get_duplicate_method_report;
+    $report .= $self->_get_long_method_report;
+
+    if ($report) {
+        my $target = $self->target_class;
+        $report = "Report for class: $target\n\n$report";
     }
+    return $report;
+}
+
+sub _get_duplicate_method_report {
+    my $self = shift;
+
+    my $report = '';
+    my @duplicate = $self->duplicate_methods;
+    my (@methods, @duplicates);
+    if ( @duplicate ) {
+        foreach my $duplicate (@duplicate) {
+            push @methods => join '::' => @{ pop @$duplicate };
+            push @duplicates => join "\n" => map { join '::' => @$_ } @$duplicate;
+        }
+        $report .= "Duplicate Methods (Experimental)\n"
+          . $self->_build_report( 'Method', 'Duplicated In', \@methods,
+          \@duplicates );
+    }
+    return $report;
+}
+
+sub _get_overridden_report {
+    my $self = shift;
+
+    my $report = '';
+    my $overridden = $self->overridden;
+    if ( %$overridden ) {
+        my @methods = sort keys %$overridden;
+        my @classes;
+        foreach my $method (@methods) {
+            push @classes => join "\n" => @{ $overridden->{$method} };
+        }
+        $report .= "Overridden Methods\n"
+          . $self->_build_report( 'Method', 'Class', \@methods, \@classes );
+    }
+    return $report;
+}
+
+sub _get_unreachable_report {
+    my $self = shift;
+
+    my $report = '';
+    if ( my @unreachable = $self->unreachable ) {
+        my ( @methods, @classes );
+        for my $fq_method (@unreachable) {
+            $fq_method =~ /^(.*)::(.*)$/;    # time to rethink the API
+            push @methods => $2;
+            push @classes => $1;
+        }
+        $report .= "Unreachable Methods\n"
+          . $self->_build_report( 'Method', 'Class', \@methods, \@classes );
+    }
+    return $report;
+}
+
+sub _get_multiple_inheritance_report {
+    my $self = shift;
+    my $report .= '';
+    if ( my @multis = $self->multiple_inheritance ) {
+        my @classes = map { join "\n" => $self->parents($_) } @multis;
+        $report .= "Multiple Inheritance\n"
+          . $self->_build_report( 'Class', 'Parents', \@multis, \@classes );
+    }
+    return $report;
+}
+
+sub _get_exported_report {
+    my $self = shift;
+    my $exported = $self->exported;
+    my $report = '';
+    if ( my @classes = sort keys %$exported ) {
+        my ($longest_c, $longest_m) = (length('Class'), length('Method') );
+        my (@subs,@sources);
+        foreach my $class (@classes) {
+            my (@temp_subs, @temp_sources);
+            foreach my $sub (sort keys %{ $exported->{$class} } ) {
+                push @temp_subs => $sub;
+                push @temp_sources => $exported->{$class}{$sub};
+                $longest_c = length($class) if length($class) > $longest_c;
+                $longest_m = length($sub)   if length($sub) > $longest_m;
+            }
+            push @subs    => join "\n" => @temp_subs;
+            push @sources => join "\n" => @temp_sources;
+        }
+        my $width = $self->width - 3;
+        my $third = int($width/3);
+        $longest_c = $third if $longest_c > $third;
+        $longest_m = $third if $longest_m > $third;
+        my $rest = $width - ($longest_c + $longest_m);
+        my $text = Text::SimpleTable->new(
+            [ $longest_c, 'Class' ],
+            [ $longest_m, 'Method' ],
+            [ $rest,      'Exported From Package' ]
+        );
+        for my $i ( 0 .. $#classes ) {
+            $text->row( $classes[$i], $subs[$i], $sources[$i] );
+        }
+        $report .= "Exported Subroutines\n".$text->draw;
+    }
+    return $report;
+}
+
+sub _get_long_method_report {
+    my $self = shift;
+    my $report .= '';
+    my %long_methods = $self->long_methods;
+    if ( my @methods = sort keys %long_methods ) {
+        my @lengths;
+        foreach my $method (@methods) {
+            push @lengths => $long_methods{$method};
+        }
+        $report .= "Long Methods (experimental)\n"
+          . $self->_build_report( 'Method', 'Approximate Length',
+            \@methods, \@lengths );
+    }
+    return $report;
+}
+
+sub _build_report {
+    my ( $self, $title1, $title2, $strings1, $strings2 ) = @_;
+    unless ( @$strings1 == @$strings2 ) {
+        Carp::croak("PANIC:  Attempt to build unbalanced report");
+    }
+    my ( $width1, $width2 ) = $self->_get_widths( $title1, @$strings1 );
+    my $text =
+      Text::SimpleTable->new( [ $width1, $title1 ], [ $width2, $title2 ] );
+    for my $i ( 0 .. $#$strings1 ) {
+        $text->row( $strings1->[$i], $strings2->[$i] );
+    }
+    return $text->draw;
+}
+
+sub _get_widths {
+    my ( $self, $title, @strings ) = @_;
+
+    my $width = $self->width;
+    my $longest = length($title);
+    foreach my $string (@strings) {
+        my $length = length $string;
+        $longest = $length if $length > $longest;
+    }
+    $longest = int( $width / 2 ) if $longest > ($width / 2);
+    return ($longest, $width - $longest);
+}
+
+=head2 C<width>
+
+ $sniff->width(80);
+
+Set the width of the report.  Defaults to 72.
+
+=cut
+
+sub width {
+    my $self = shift;
+    return $self->{width} unless @_;
+    my $number = shift;
+    unless ( $number =~ /^\d+$/ && $number >= 40 ) {
+        Carp::croak(
+            "Argument to 'width' must be a number >= than 40, not ($number)");
+    }
+    $self->{width} = $number;
 }
 
 =head2 C<to_string>
@@ -726,6 +833,17 @@ This is the class you originally asked to sniff.
 
 sub target_class { $_[0]->{target} }
 
+=head2 C<method_length>
+
+ my $method_length = $sniff->method_length;
+
+This is the maximum allowed length of a method before being reported as a code
+smell.  See C<method_length> in the constructor.
+
+=cut
+
+sub method_length { $_[0]->{method_length} }
+
 =head2 C<ignore>
 
  my $ignore = $sniff->ignore;
@@ -762,35 +880,6 @@ In list context, lists the classes in the hierarchy, in default search order.
 
 sub classes { @{ $_[0]->{list_classes} } }
 
-=head2 C<parents>
-
- # defaults to 'target_class'
- my $num_parents = $sniff->parents;
- my @parents     = $sniff->parents;
-
- my $num_parents = $sniff->parents('Some::Class');
- my @parents     = $sniff->parents('Some::Class');
-
-In scalar context, lists the number of parents a class has.
-
-In list context, lists the parents a class has.
-
-=head3 Code Smell:  multiple parens (multiple inheritance)
-
-If a class has more than one parent, you may have unreachable or conflicting
-methods.
-
-=cut
-
-sub parents {
-    my ( $self, $class ) = @_;
-    $class ||= $self->target_class;
-    unless ( exists $self->{classes}{$class} ) {
-        Carp::croak("No such class '$class' found in hierarchy");
-    }
-    return @{ $self->{classes}{$class}{parents} };
-}
-
 =head2 C<children>
 
  # defaults to 'target_class'
@@ -840,7 +929,8 @@ sub methods {
 }
 
 sub _get_parents {
-    my ( $self, $class ) = @_;
+    my ( $self, $node ) = @_;
+    my $class = $node->value;
     return if $class eq 'UNIVERSAL';
     no strict 'refs';
 
@@ -854,27 +944,18 @@ sub _get_parents {
     return @parents;
 }
 
+# This is the heart of where we set just about everything up.
 sub _build_tree {
     my ( $self, @nodes ) = @_;
 
     for my $node (@nodes) {
-        my $class = $node->value;
-
-        my @parents = $self->_get_parents($class) or return;
-
-        $self->_build_paths( $class, @parents );
-        $self->_add_relationships( $class, @parents );
-
-        # This algorithm will follow classes in Perl's default inheritance
-        # order
-        foreach my $parent (@parents) {
-            push @{ $self->{list_classes} } => $parent
-              unless grep { $_ eq $parent } @{ $self->{list_classes} };
-            $self->{classes}{$parent}{count}++;
-            my $tree = Tree->new($parent);
-            $node->add_child($tree);
-            $self->_build_tree($tree);
+        return unless $self->_get_parents($node);
+        foreach my $class ( $node->value, $self->_get_parents($node) ) {
+            $self->_register_class($class);
         }
+        $self->_add_children($node);
+        $self->_build_paths($node);
+        $self->_add_parents($node);
     }
 }
 
@@ -882,35 +963,73 @@ sub _build_tree {
 # will take through the code to find a method.  This is based on Perl's
 # default search order, not C3.
 sub _build_paths {
-    my ( $self, $class, @parents ) = @_;
+    my ( $self, $node) = @_;
 
+    my $class = $node->value;
+    my @parents = $self->_get_parents($node);
+    
     # XXX strictly speaking, we can skip $do_chg, but if path() get's
-    # expensive (such as testing for valid classes or circularity), then we
+    # expensive (such as testing for valid classes), then we
     # need it.
     my $do_chg;
+    my @paths;
 
-    my @paths = map {
-        my $path = $_;
-        $path->[-1] eq $class
-          ? do {
+    foreach my $path ( $self->paths ) {
+        if ( $path->[-1] eq $class ) {
+            foreach my $parent (@parents) {
+                if ( grep { $parent eq $_ } @$path ) {
+                    my $circular = join ' -> ' => @$path, $parent;
+                    Carp::croak("Circular path found in path ($circular)");
+                }
+            }
             ++$do_chg;
-            map { [ @$path, $_ ] } @parents;
-          }
-          : $path
-    } $self->paths;
+            push @paths => map { [ @$path, $_ ] } @parents;
+        }
+        else {
+            push @paths => $path;
+        }
+    }
 
     $self->paths(@paths) if $do_chg;
 }
 
+sub _add_parents {
+    my ( $self, $node ) = @_;
+
+    # This algorithm will follow classes in Perl's default inheritance
+    # order
+    foreach my $parent ($self->_get_parents($node)) {
+        push @{ $self->{list_classes} } => $parent
+          unless grep { $_ eq $parent } @{ $self->{list_classes} };
+        $self->{classes}{$parent}{count}++;
+        my $tree = Tree->new($parent);
+        $node->add_child($tree);
+        $self->_build_tree($tree);
+    }
+}
+sub _add_children {
+    my ( $self, $node ) = @_;
+    my $class   = $node->value;
+    my @parents = $self->_get_parents($node);
+
+    $self->{classes}{$class}{parents} = \@parents;
+    $self->_add_child( $_, $class ) foreach @parents;
+    return $self;
+}
+
+sub _add_child {
+    my ( $self, $class, $child ) = @_;
+
+    my $children = $self->{classes}{$class}{children};
+    unless ( grep { $child eq $_ } @$children ) {
+        push @$children => $child;
+    }
+}
+
 =head1 CAVEATS AND PLANS
 
 =over 4
 
-=item * Circular Inheritance
-
-Currently, any circular inheritances causes a 'Deep recursion' failure, so
-don't do that.
-
 =item * Package Variables
 
 User-defined package variables in OO code are a code smell, but with versions
@@ -923,16 +1042,6 @@ future (there will be exceptions, such as with @ISA).
 I'd like support for alternate method resolution orders.  If your classes use
 C3, you may get erroneous results.  See L<paths> for a workaround.
 
-=item * Exporting
-
-Many packages (such as L<Data::Dumper>) export functions by default and these
-show up as methods.  We'll detect those later.
-
-=item * Duplicate Methods
-
-It's rather common for someone to cut-n-paste a method from one class to
-another.  We'll try and detect that, too.  L<Sub::Information> may help there.
-
 =back
 
 =head1 AUTHOR
